﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Configuration;

namespace Snmp2Mysql
{
	class Snmp2MysqlException : Exception
	{
		public Snmp2MysqlException(string msg) : base(msg) {}
		public Snmp2MysqlException(string msg, Exception rootCause) : base(msg, rootCause) { }
	}


    class Program
    {
        static void Main(string[] args)
        {
			//string loc = AppDomain.CurrentDomain.SetupInformation.ConfigurationFile;
			//Console.WriteLine ("loc: {0}", loc);

			string dbConnStr = ConfigurationManager.AppSettings["DbConnStr"];
			if (dbConnStr == null || "".Equals (dbConnStr)) {
				throw new Snmp2MysqlException ("dbConnStr not found in configuration");
			}
			string verboseStr = ConfigurationManager.AppSettings ["Verbose"];
			if (verboseStr == null || "".Equals (verboseStr)) {
				throw new Snmp2MysqlException ("Verbose not found in configuration");
			}
			bool verbose = verboseStr.ToUpper().Equals ("TRUE");

			if (verbose)
				Console.WriteLine ("Database connection to {0}", dbConnStr);
			
            DatabaseLink dbLink = new DatabaseLink(dbConnStr);
            using (DataCollector dc = dbLink.DataCollector)
            using (DeviceProvider dp = dbLink.DeviceProvider)
            {
                foreach (DeviceTuple dt in dp)
                {
					if (verbose) Console.WriteLine("dt: {0}, {1}", dt.DeviceAddress, dt.Community);

                    using (SnmpGetter snmpGetter = new SnmpGetter(dt.Community, dt.DeviceAddress))
                    {
                        using (DeviceDataItemProvider ddip = dt.DeviceDataItemProvider)
                        {
                            foreach (DeviceDataItemTuple ddit in dt.DeviceDataItemProvider)
                            {
								if (verbose) Console.WriteLine("  ddit: {0}, {1}", ddit.Id, ddit.Oid);
                                snmpGetter.AddOid(ddit.Id, ddit.Oid);
                            }
                        }

						try 
						{
                        	SnmpGetterResultProvider res = snmpGetter.Exec();
                        	foreach (SnmpGetterResult r in res)
                        	{
								if (verbose) Console.WriteLine("{0} {1} {2}: {3}", r.Index, r.Oid, r.Type, r.Value);
                            	dc.add((int)r.Index, r.Value);
                        	}
						} 
						catch (SnmpGetterException e) {
							if (verbose) Console.WriteLine ("no result: {0}", e.Message);
						}
                    }
                }
            }
        }
    }
}
